@@ -2,15 +2,18 @@ __copyright__ = "Copyright (c) 2020 Jina AI Limited. All rights reserved."
 __license__ = "Apache-2.0"
 
 import glob
-import tempfile
+import json
 import urllib.parse
+import urllib.request
 import webbrowser
 from typing import Dict
 
 from .checker import *
-from .helper import get_default_login
+from .database import MongoDBHandler
+from .helper import handle_dot_in_keys
 from ..clients.python import ProgressBar
-from ..helper import colored, get_readable_size, get_now_timestamp
+from ..excepts import PeaFailToStart
+from ..helper import colored, get_readable_size, get_now_timestamp, get_full_version, random_name, expand_dict
 from ..logging import get_logger
 from ..logging.profile import TimeContext
 
@@ -50,7 +53,7 @@ class HubIO:
             self.logger.critical('requires "docker" dependency, please install it via "pip install jina[docker]"')
             raise
 
-    def new(self):
+    def new(self) -> None:
         """Create a new executor using cookiecutter template """
         try:
             from cookiecutter.main import cookiecutter
@@ -58,11 +61,45 @@ class HubIO:
             self.logger.critical('requires "cookiecutter" dependency, please install it via "pip install cookiecutter"')
             raise
 
-        cookiecutter(self.args.template, overwrite_if_exists=self.args.overwrite, output_dir=self.args.output_dir)
+        import click
+        cookiecutter_template = self.args.template
+        if self.args.type == 'app':
+            cookiecutter_template = 'https://github.com/jina-ai/cookiecutter-jina.git'
+        elif self.args.type == 'pod':
+            cookiecutter_template = 'https://github.com/jina-ai/cookiecutter-jina-hub.git'
+        cookiecutter(cookiecutter_template, overwrite_if_exists=self.args.overwrite, output_dir=self.args.output_dir)
 
-    def push(self, name: str = None, readme_path: str = None):
-        """A wrapper of docker push """
+        try:
+            cookiecutter(cookiecutter_template, overwrite_if_exists=self.args.overwrite,
+                         output_dir=self.args.output_dir)
+        except click.exceptions.Abort:
+            self.logger.info('nothing is created, bye!')
+
+    def push(self, name: str = None, readme_path: str = None) -> None:
+        """ A wrapper of docker push 
+        - Checks for the tempfile, returns without push if it cannot find
+        - Pushes to docker hub, returns withput writing to db if it fails
+        - Writes to the db
+        """
         name = name or self.args.name
+        file_path = get_summary_path(name)
+        if not os.path.isfile(file_path):
+            self.logger.error(f'can not find the build summary file')
+            return
+
+        try:
+            self._push_docker_hub(name, readme_path)
+        except:
+            self.logger.error('can not push to the docker hub registry')
+            return
+
+        with open(file_path) as f:
+            result = json.load(f)
+        if result['is_build_success']:
+            self._write_summary_to_db(summary=result)
+
+    def _push_docker_hub(self, name: str = None, readme_path: str = None) -> None:
+        """ Helper push function """
         check_registry(self.args.registry, name, _repo_prefix)
         self._check_docker_image(name)
         self.login()
@@ -75,10 +112,9 @@ class HubIO:
         if False and readme_path:
             # unfortunately Docker Hub Personal Access Tokens cannot be used as they are not supported by the API
             _volumes = {os.path.dirname(os.path.abspath(readme_path)): {'bind': '/workspace'}}
-            _env = get_default_login()
             _env = {
-                'DOCKERHUB_USERNAME': _env['username'],
-                'DOCKERHUB_PASSWORD': _env['password'],
+                'DOCKERHUB_USERNAME': self.args.username,
+                'DOCKERHUB_PASSWORD': self.args.password,
                 'DOCKERHUB_REPOSITORY': name.split(':')[0],
                 'README_FILEPATH': '/workspace/README.md',
             }
@@ -98,19 +134,24 @@ class HubIO:
             self.logger.info(
                 f'Check out the usage {colored(share_link, "cyan", attrs=["underline"])} and share it with others!')
 
-    def pull(self):
+    def pull(self) -> None:
         """A wrapper of docker pull """
         check_registry(self.args.registry, self.args.name, _repo_prefix)
         self.login()
-        with TimeContext(f'pulling {self.args.name}', self.logger):
-            image = self._client.images.pull(self.args.name)
-        if isinstance(image, list):
-            image = image[0]
-        image_tag = image.tags[0] if image.tags else ""
-        self.logger.success(
-            f'🎉 pulled {image_tag} ({image.short_id}) uncompressed size: {get_readable_size(image.attrs["Size"])}')
+        try:
+            with TimeContext(f'pulling {self.args.name}', self.logger):
+                image = self._client.images.pull(self.args.name)
+            if isinstance(image, list):
+                image = image[0]
+            image_tag = image.tags[0] if image.tags else ''
+            self.logger.success(
+                f'🎉 pulled {image_tag} ({image.short_id}) uncompressed size: {get_readable_size(image.attrs["Size"])}')
+        except:
+            self.logger.error(f'can not pull image {self.args.name} from {self.args.registry}')
+            raise
 
-    def _check_docker_image(self, name: str):
+
+    def _check_docker_image(self, name: str) -> None:
         # check local image
         image = self._client.images.get(name)
         for r in _allowed:
@@ -127,19 +168,13 @@ class HubIO:
 
         self.logger.info(f'✅ {name} is a valid Jina Hub image, ready to publish')
 
-    def login(self):
+    def login(self) -> None:
         """A wrapper of docker login """
-        try:
-            password = self.args.password  # or (self.args.password_stdin and self.args.password_stdin.read())
-        except ValueError:
-            password = ''
-
-        if self.args.username and password:
-            self._client.login(username=self.args.username, password=password,
+        if self.args.username and self.args.password:
+            self._client.login(username=self.args.username, password=self.args.password,
                                registry=self.args.registry)
         else:
-            # use default login
-            self._client.login(**get_default_login(), registry=self.args.registry)
+            raise ValueError('no username/password specified, docker login failed')
 
     def build(self) -> Dict:
         """A wrapper of docker build """
@@ -157,7 +192,7 @@ class HubIO:
                     streamer = self._raw_client.build(
                         decode=True,
                         path=self.args.path,
-                        tag=self.canonical_name,
+                        tag=self.tag,
                         pull=self.args.pull,
                         dockerfile=self.dockerfile_path_revised,
                         rm=True
@@ -182,7 +217,7 @@ class HubIO:
             if is_build_success:
                 # compile it again, but this time don't show the log
                 image, log = self._client.images.build(path=self.args.path,
-                                                       tag=self.canonical_name,
+                                                       tag=self.tag,
                                                        pull=self.args.pull,
                                                        dockerfile=self.dockerfile_path_revised,
                                                        rm=True)
@@ -203,12 +238,39 @@ class HubIO:
                 self.logger.error(f'can not build the image, please double check the log')
                 _details = {}
 
-            if is_build_success and self.args.push:
-                try:
-                    self.push(image.tags[0], self.readme_path)
-                    is_push_success = True
-                except Exception:
-                    self.logger.error(f'can not push tot the registry')
+            if is_build_success:
+                if self.args.test_uses:
+                    try:
+                        is_build_success = False
+                        from jina.flow import Flow
+                        p_name = random_name()
+                        with Flow().add(name=p_name, uses=image.tags[0], daemon=self.args.daemon):
+                            pass
+                        if self.args.daemon:
+                            self._raw_client.stop(p_name)
+                        self._raw_client.prune_containers()
+                        is_build_success = True
+                    except PeaFailToStart:
+                        self.logger.error(f'can not use it in the Flow, please check your file bundle')
+                    except Exception as ex:
+                        self.logger.error(f'something wrong but it is probably not your fault. {repr(ex)}')
+
+                _version = self.manifest['version'] if 'version' in self.manifest else '0.0.1'
+                info, env_info = get_full_version()
+                _host_info = {
+                    'jina': info,
+                    'jina_envs': env_info,
+                    'docker': self._raw_client.info(),
+                    'build_args': vars(self.args)
+                }
+
+            _build_history = {
+                'time': get_now_timestamp(),
+                'host_info': _host_info if is_build_success and self.args.host_info else '',
+                'duration': tc.readable_duration,
+                'logs': _logs,
+                'exception': _excepts
+            }
 
             if self.args.prune_images:
                 self.logger.info('deleting unused images')
@@ -216,21 +278,31 @@ class HubIO:
 
             result = {
                 'name': getattr(self, 'canonical_name', ''),
+                'version': self.manifest['version'] if is_build_success and 'version' in self.manifest else '0.0.1',
                 'path': self.args.path,
+                'manifest_info': self.manifest if is_build_success else '',
                 'details': _details,
-                'last_build_time': get_now_timestamp(),
-                'build_duration': tc.duration,
                 'is_build_success': is_build_success,
-                'is_push_success': is_push_success,
-                'build_logs': _logs,
-                'exception': _excepts
+                'build_history': [_build_history]
             }
+
+            # only successful build (NOT dry run) writes the summary to disk
+            if result['is_build_success']:
+                self._write_summary_to_file(summary=result)
+                if self.args.push:
+                    try:
+                        self._push_docker_hub(image.tags[0], self.readme_path)
+                        self._write_summary_to_db(summary=result)
+                        self._write_slack_message(result, _details, _build_history)
+                    except Exception as ex:
+                        self.logger.error(f'can not complete the push due to {repr(ex)}')
+
         if not result['is_build_success'] and self.args.raise_error:
             # remove the very verbose build log when throw error
-            result.pop('build_logs')
+            result['build_history'][0].pop('logs')
             raise RuntimeError(result)
-        else:
-            return result
+
+        return result
 
     def dry_run(self) -> Dict:
         try:
@@ -241,6 +313,36 @@ class HubIO:
                  'exception': str(ex)}
         return s
 
+    def _write_summary_to_db(self, summary: Dict) -> None:
+        """ Inserts / Updates summary document in mongodb """
+        if not is_db_envs_set():
+            self.logger.warning('MongoDB environment vars are not set! bookkeeping skipped.')
+            return
+
+        build_summary = handle_dot_in_keys(document=summary)
+        _build_query = {'name': build_summary['name'], 'version': build_summary['version']}
+        _current_build_history = build_summary['build_history']
+        with MongoDBHandler(hostname=os.environ['JINA_DB_HOSTNAME'],
+                            username=os.environ['JINA_DB_USERNAME'],
+                            password=os.environ['JINA_DB_PASSWORD'],
+                            database_name=os.environ['JINA_DB_NAME'],
+                            collection_name=os.environ['JINA_DB_COLLECTION']) as db:
+            existing_doc = db.find(query=_build_query)
+            if existing_doc:
+                build_summary['build_history'] = existing_doc['build_history'] + _current_build_history
+                _modified_count = db.replace(document=build_summary,
+                                             query=_build_query)
+                self.logger.debug(f'Updated the build + push summary in db. {_modified_count} documents modified')
+            else:
+                _inserted_id = db.insert(document=build_summary)
+                self.logger.debug(f'Inserted the build + push summary in db with id {_inserted_id}')
+
+    def _write_summary_to_file(self, summary: Dict) -> None:
+        file_path = get_summary_path(f'{summary["name"]}:{summary["version"]}')
+        with open(file_path, 'w+') as f:
+            json.dump(summary, f)
+        self.logger.debug(f'stored the summary from build to {file_path}')
+
     def _check_completeness(self) -> Dict:
         self.dockerfile_path = get_exist_path(self.args.path, 'Dockerfile')
         self.manifest_path = get_exist_path(self.args.path, 'manifest.yml')
@@ -276,12 +378,13 @@ class HubIO:
             self.logger.critical('Dockerfile or manifest.yml is not given, can not build')
             raise FileNotFoundError('Dockerfile or manifest.yml is not given, can not build')
 
-        tmp = self._read_manifest(self.manifest_path)
-        self.dockerfile_path_revised = self._get_revised_dockerfile(self.dockerfile_path, tmp)
-        self.canonical_name = safe_url_name(f'{_repo_prefix}' + '{type}.{kind}.{name}:{version}'.format(**tmp))
+        self.manifest = self._read_manifest(self.manifest_path)
+        self.dockerfile_path_revised = self._get_revised_dockerfile(self.dockerfile_path, self.manifest)
+        self.tag = safe_url_name(f'{_repo_prefix}' + '{type}.{kind}.{name}:{version}'.format(**self.manifest))
+        self.canonical_name = safe_url_name(f'{_repo_prefix}' + '{type}.{kind}.{name}'.format(**self.manifest))
         return completeness
 
-    def _read_manifest(self, path: str, validate: bool = True):
+    def _read_manifest(self, path: str, validate: bool = True) -> Dict:
         with resource_stream('jina', '/'.join(('resources', 'hub-builder', 'manifest.yml'))) as fp:
             tmp = yaml.load(fp)  # do not expand variables at here, i.e. DO NOT USE expand_dict(yaml.load(fp))
 
@@ -293,7 +396,7 @@ class HubIO:
 
         return tmp
 
-    def _validate_manifest(self, manifest: Dict):
+    def _validate_manifest(self, manifest: Dict) -> None:
         required = {'name', 'type', 'version'}
 
         # check the required field in manifest
@@ -323,14 +426,12 @@ class HubIO:
         for k, v in manifest.items():
             if v and isinstance(v, str):
                 manifest[k] = remove_control_characters(v)
-            elif v and isinstance(v, list):
-                manifest[k] = ','.join(v)
 
         # show manifest key-values
         for k, v in manifest.items():
             self.logger.debug(f'{k}: {v}')
 
-    def _get_revised_dockerfile(self, dockerfile_path: str, manifest: Dict):
+    def _get_revised_dockerfile(self, dockerfile_path: str, manifest: Dict) -> str:
         # modify dockerfile
         revised_dockerfile = []
         with open(dockerfile_path) as fp:
@@ -348,3 +449,28 @@ class HubIO:
         for k in revised_dockerfile:
             self.logger.debug(k)
         return f
+
+    def _write_slack_message(self, *args):
+        def _expand_fn(v):
+            if isinstance(v, str):
+                for d in args:
+                    try:
+                        v = v.format(**d)
+                    except KeyError:
+                        pass
+            return v
+
+        if 'JINAHUB_SLACK_WEBHOOK' in os.environ:
+            with resource_stream('jina', '/'.join(('resources', 'hub-builder-success', 'slack-jinahub.json'))) as fp:
+                tmp = expand_dict(json.load(fp), _expand_fn, resolve_cycle_ref=False)
+                req = urllib.request.Request(os.environ['JINAHUB_SLACK_WEBHOOK'])
+                req.add_header('Content-Type', 'application/json; charset=utf-8')
+                jdb = json.dumps(tmp).encode('utf-8')  # needs to be bytes
+                req.add_header('Content-Length', str(len(jdb)))
+                with urllib.request.urlopen(req, jdb) as f:
+                    res = f.read()
+                    self.logger.info(f'push to Slack: {res}')
+
+    # alias of "new" in cli
+    create = new
+    init = new
