@@ -25,18 +25,18 @@ class EncodeDriver(BaseEncodeDriver):
         super().__init__(*args, **kwargs)
         self._is_apply = False
 
-    def _apply_all(self, docs: Iterable['jina_pb2.Document'], *args, **kwargs):
+    def _apply_all(self, docs: Iterable['jina_pb2.Document'], *args, **kwargs) -> None:
         contents, docs_pts, bad_doc_ids = extract_docs(docs, embedding=False)
 
         if bad_doc_ids:
             self.logger.warning(f'these bad docs can not be added: {bad_doc_ids} '
-                                f'from level depth {docs[0].level_depth}')
+                                f'from level depth {docs[0].granularity}')
 
         if docs_pts:
             embeds = self.exec_fn(contents)
             if len(docs_pts) != embeds.shape[0]:
                 self.logger.error(
-                    f'mismatched {len(docs_pts)} docs from level {docs[0].level_depth} '
+                    f'mismatched {len(docs_pts)} docs from level {docs[0].granularity} '
                     f'and a {embeds.shape} shape embedding, the first dimension must be the same')
             for doc, embedding in zip(docs_pts, embeds):
                 doc.embedding.CopyFrom(array2pb(embedding))
