@@ -1,7 +1,7 @@
 __copyright__ = "Copyright (c) 2020 Jina AI Limited. All rights reserved."
 __license__ = "Apache-2.0"
 
-from typing import Iterable
+from typing import Iterable, List
 
 from .. import QuerySetReader, BaseRecursiveDriver
 from ...helper import rgetattr
@@ -17,14 +17,12 @@ class SortQL(QuerySetReader, BaseRecursiveDriver):
         Example::
         - !ReduceAllDriver
             with:
-                granularity_range: [0, 0]
-                adjacency_range: [0, 1]
+                traversal_paths: ['m']
         - !SortQL
             with:
                 reverse: true
                 field: 'score.value'
-                granularity_range: [0, 0]
-                adjacency_range: [0, 1]
+                traversal_paths: ['m']
         - !SliceQL
             with:
                 start: 0
@@ -35,13 +33,13 @@ class SortQL(QuerySetReader, BaseRecursiveDriver):
         `SortQL` will ensure that only the documents are sorted by the score value before slicing the first top 50 documents
     """
 
-    def __init__(self, field: str, reverse: bool = False, *args, **kwargs):
+    def __init__(self, field: str, reverse: bool = False, traversal_paths: List[str] = ['c'], *args, **kwargs):
         """
         :param field: the value of the field drives the sort of the iterable docs
         :param reverse: sort the value from big to small
         """
 
-        super().__init__(*args, **kwargs)
+        super().__init__(use_tree_traversal=True, traversal_paths=traversal_paths, *args, **kwargs)
         self._reverse = reverse
         self._field = field
         self.is_apply = False
