@@ -63,7 +63,8 @@ def hello_world(args):
     # run it!
     with f:
         f.search_ndarray(targets['query']['data'], shuffle=True, size=args.num_query,
-                         output_fn=print_result, batch_size=args.query_batch_size)
+                         output_fn=print_result, batch_size=args.query_batch_size,
+                         top_k=args.top_k)
 
     # write result to html
     write_html(os.path.join(args.workdir, 'hello-world.html'))
